import React, { useState } from 'react';
import { Text, Box, Newline, Spacer, useInput } from 'ink';
import TextInput from 'ink-text-input';
import execall from 'execall';
import RangeStepper from 'range-stepper';
const defaultText = 'My sample string\nto be tested against\nsome regular expressions';
const defaultFlags = 'gm';
const panelColors = {
    active: '#a0fc02',
    common: '#f0f0fc',
};
function getErrorMessage(error) {
    if (error instanceof Error)
        return error.message;
    return String(error);
}
export default function App({ source = defaultText, regTxtVal: regTxtValue = '', }) {
    const getHighlightedMatches = (source, rgxTxt, flags = 'gm') => {
        if (regTxt === '') {
            return source;
        }
        const result = execall(new RegExp(rgxTxt, flags), source);
        let j = 0;
        const acc = result.map(curr => {
            const plainText = React.createElement(Text, null, source.slice(j, curr.index));
            const highlightedText = (React.createElement(Text, { color: "#101820", backgroundColor: "#FEE715" }, source.slice(curr.index, curr.index + curr.match.length)));
            j = curr.index + curr.match.length;
            return (React.createElement(Text, { key: curr.index },
                plainText,
                highlightedText));
        });
        if (j < source.length) {
            return (React.createElement(Text, null,
                acc,
                React.createElement(Text, null, source.slice(j, source.length))));
        }
        return acc;
    };
    const [txt, setTxt] = useState(source);
    const [regTxt, setRegTxt] = useState(regTxtValue);
    const [regFlags, setRegFlags] = useState(defaultFlags);
    const [textEditing, setTextEditing] = useState(false);
    const [regIsValid, setRegIsValid] = useState(true);
    const [regError, setRegError] = useState('');
    const [submitted, setSubmitted] = useState(regTxt !== '');
    const [lastTxt, setLastTxt] = useState(source);
    const [activeInputIndex, setActiveInputIndex] = useState(0);
    const [lastHighlightedText, setLastHighlightedText] = useState(regTxt === '' || !submitted
        ? txt
        : getHighlightedMatches(txt, regTxt, regFlags));
    const [lastFlags, setLastFlags] = useState(defaultFlags);
    const [panelStepper, setPanelStepper] = useState(new RangeStepper({ max: 2, current: 1 }));
    function GetStatusText() {
        if (!regIsValid) {
            return `Bad reg expression: ${regError}`;
        }
        if (textEditing) {
            return 'Editing the source text';
        }
        if (panelStepper.isCurrent(0)) {
            return 'Selected source text panel';
        }
        if (regTxt !== '' && !submitted) {
            return activeInputIndex === 0 ? 'Typing regexp' : 'Typing flags';
        }
        if (submitted) {
            return txt === lastTxt
                ? 'Showing highlights for specified regexp'
                : activeInputIndex === 0
                    ? 'Typing regexp '
                    : 'Typing flags';
        }
        return '';
    }
    useInput((input, key) => {
        if (key.tab) {
            if (!panelStepper.isCurrent(0))
                setActiveInputIndex(activeInputIndex === 1 ? 0 : 1);
            setPanelStepper(panelStepper.next().dup());
            setTextEditing(false);
        }
        else if (input === 'i' && !textEditing && panelStepper.isCurrent(0)) {
            setLastHighlightedText(defaultText);
            setTextEditing(true);
        }
    });
    const getPanelColor = (i) => panelStepper.isCurrent(i) ? panelColors.active : panelColors.common;
    return (React.createElement(Box, { flexDirection: "column" },
        React.createElement(Box, null,
            React.createElement(Text, null, "Status: "),
            React.createElement(Text, null,
                React.createElement(GetStatusText, null)),
            React.createElement(Spacer, null),
            React.createElement(Text, null, panelStepper.isCurrent(0) && !textEditing
                ? 'Type `i` to start editing source text'
                : '<tab> - activates the next section')),
        React.createElement(Box, { borderStyle: "single", borderColor: getPanelColor(0) },
            React.createElement(Newline, null),
            textEditing ? (React.createElement(TextInput, { focus: panelStepper.isCurrent(0), value: lastTxt, onChange: setLastTxt, onSubmit: value => {
                    setLastTxt(value);
                    setLastHighlightedText(getHighlightedMatches(value, regTxt, regFlags));
                    setPanelStepper(panelStepper.next().dup());
                    setTextEditing(false);
                } })) : (React.createElement(Text, null, lastHighlightedText))),
        React.createElement(Box, { borderStyle: "single", borderColor: getPanelColor(1) },
            React.createElement(Text, { italic: panelStepper.isCurrent(1), color: regIsValid ? '' : 'red' },
                "Regexp:",
                ' '),
            React.createElement(TextInput, { focus: panelStepper.isCurrent(1), value: regTxt, onChange: value => {
                    setSubmitted(false);
                    setRegTxt(value);
                }, onSubmit: value => {
                    let isValid = true;
                    try {
                        new RegExp(value, regFlags); // eslint-disable-line no-new
                    }
                    catch (error) {
                        isValid = false;
                        setRegError(getErrorMessage(error));
                        setRegIsValid(false);
                    }
                    if (!isValid) {
                        return;
                    }
                    if (regError !== '') {
                        setRegError('');
                    }
                    if (!regIsValid) {
                        setRegIsValid(true);
                    }
                    setTxt(lastTxt);
                    setLastHighlightedText(getHighlightedMatches(lastTxt, value, regFlags));
                    setRegTxt(value);
                    setSubmitted(true);
                } }),
            React.createElement(Spacer, null),
            React.createElement(Text, null,
                React.createElement(Text, { italic: panelStepper.isCurrent(2) }, "Flags: "),
                React.createElement(TextInput, { focus: panelStepper.isCurrent(2), value: regFlags, onChange: value => {
                        setSubmitted(false);
                        const hasDup = [...value].some((v, i, a) => a.lastIndexOf(v) !== i);
                        if (hasDup) {
                            setRegFlags(lastFlags);
                            return;
                        }
                        for (const flag of value) {
                            if (!'gimsuy'.includes(flag)) {
                                setRegFlags(lastFlags);
                                return;
                            }
                        }
                        setLastFlags(value);
                        setRegFlags(value);
                    }, onSubmit: value => {
                        setSubmitted(true);
                        setLastHighlightedText(getHighlightedMatches(lastTxt, regTxt, value));
                    } })))));
}
